// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.util.HashMap;
import java.util.List;
import java.util.LinkedList;
import java.io.Serializable;
import cexer.util.TemplateSelectionModel;
import cexer.util.Logging;
import java.io.BufferedReader;
import java.io.StringReader;
import cexer.util.ExerciseDescriptionParser;

/**
 * This is a template of an exercise that includes all basic information about the
 * exercise.
 */
public class ExerciseTemplate implements Serializable {
    // name of exercise to be shown on the web page
    private String name;
    private HashMap names;
    // exercise description XXX localization!
    private String description;
    private HashMap descriptions;
    
    // parsed (and localized) exercise descriptions:
    private String task;
    private HashMap localizedTasks;
    private String instructions;
    private HashMap localizedInstructions;
    private HashMap codeBlocks;
    private LinkedList codeBlockNames;
    private HashMap localizedCodeBlock;
    private HashMap localizedCodeBlockNames;
    
    // applet attributes
    private String codebase, archive, code;
    private int width, height;
    // link to hint page
    private String hintPage;
    // holds the parameters of the applet. Must contain only instances of AppletParameters
    private List appletParameters;
    public static final int DEFAULT_WIDTH = 800;
    public static final int DEFAULT_HEIGHT = 800;
    private int id;

    private int uses;

    public ExerciseTemplate(String[] params, List p) {
        this.name = params[0];
        this.description = params[1];
        this.setCodeBase(params[2]);
        this.archive = params[3];
        this.code = params[4];

        try {
            this.width = Integer.parseInt(params[5]);
            this.height = Integer.parseInt(params[6]);
        } catch (NumberFormatException e) {
            Logging.error(this, "Invalid width or height parameter!", e);
            if (width == 0)
            width = DEFAULT_WIDTH;
            height = DEFAULT_HEIGHT;
        }

        this.hintPage = params[7];
        this.appletParameters = p;
        uses = 0;
        descriptions = new HashMap();
        names = new HashMap();

        //Universal language description parser
        ExerciseDescriptionParser parser = new ExerciseDescriptionParser(new BufferedReader(new StringReader(description)));
        task = parser.getTask();
        codeBlocks = parser.getCodeBlocks();
        codeBlockNames = parser.getCodeBlockNames();
        instructions = parser.getInstructions();

        //HashMaps for localized versions
        localizedInstructions = new HashMap();
        localizedTasks = new HashMap();
        localizedCodeBlock = new HashMap();
        localizedCodeBlockNames = new HashMap();
    }

    public int getWidth() {
        return width;
    }

    public void setWidth(int value) {
        width = value;
    }

    public int getHeight() {
        return height;
    }

    public void setHeight(int value) {
        height = value;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String value) {
        code = value;
    }

    public String getArchive() {
        return archive;
    }

    public void setArchive(String value) {
        archive = value;
    }

    public String getCodeBase() {
        return codebase;
    }

    public void setCodeBase(String value) {
        if (value == null || value.length() == 0) {
            codebase = "https://" + System.getProperty("https.server");
        } else {
            codebase = value;
        }
    }

    public String getName() {
        return name;
    }

    public void setName(String value) {
        name = value;
    }

    public void setDescription(String value) {
        description = value;
    }

    public String getDescription() {
        return description;
    }

    public String getTask() {
        return task;
    }

    public String getInstructions() {
        return instructions;
    }

    public String getCodeBlock(String name) {
        return (String)codeBlocks.get(name);
    }

    public LinkedList getCodeBlockNames() {
        return codeBlockNames;
    }

    public void setHintPage(String value) {
        hintPage = value;
    }

    public String getHintPage() {
        return hintPage;
    }

    public void setAppletParameters(List l) {
        appletParameters = l;
    }

    public List getAppletParameters() {
        return appletParameters;
    }

    public void removeAppletParameter(AppletParameter ap) {
	if (appletParameters != null)
	    appletParameters.remove(ap);
    }

    public void addAppletParameter(AppletParameter ap) {
        if (appletParameters == null) {
            appletParameters = new LinkedList();
        }
        appletParameters.add(ap);
    }

    public void addUse() {
	uses++;
    }

    public void removeUse() {
        uses--;
    }

    public int getUses() {
        return uses;
    }

    public List getSupportedLocales() {
        return new LinkedList(names.keySet());
    }

    public void addLocalizedData(String language, String name, String description) {
        names.put(language, name);
        descriptions.put(language, description);

        ExerciseDescriptionParser parser = new ExerciseDescriptionParser(new BufferedReader(new StringReader(description)));
        localizedInstructions.put(language, parser.getInstructions());
        localizedTasks.put(language, parser.getTask());

        if (!parser.getCodeBlocks().isEmpty()) {
            localizedCodeBlock.put(language, parser.getCodeBlocks());
        }

        if (parser.getCodeBlockNames().size() > 0) {
            localizedCodeBlockNames.put(language, parser.getCodeBlockNames());
        }
    }

    public String getLocalizedName(String language) {
        if (names.containsKey(language)) {
            return (String)names.get(language);
        } else {
            return getName();
        }
    }

    public String getLocalizedDescription(String language) {
        if (descriptions.containsKey(language)) {
            return (String)descriptions.get(language);
        } else {
            return getDescription();
        }
    }

    public String getLocalizedInstructions(String language) {
        if (localizedInstructions.containsKey(language)) {
            return (String)localizedInstructions.get(language);
        } else {
            return getInstructions();
        }
    }

    public String getLocalizedTask(String language) {
        if (localizedTasks.containsKey(language)) {
            return (String)localizedTasks.get(language);
        } else {
            return getTask();
        }
    }

    public String getLocalizedCodeBlock(String blockName, String language) {
        if (localizedCodeBlock.containsKey(language)) {
            return (String)((HashMap)localizedCodeBlock.get(language)).get(blockName);
        } else {
            return getCodeBlock(blockName);
        }
    }

    public LinkedList getLocalizedCodeBlockNames(String language) {
        if (localizedCodeBlockNames.containsKey(language)) {
            return (LinkedList)localizedCodeBlockNames.get(language);
        } else {
            return getCodeBlockNames();
        }
    }

    public void setId(int value) {
        id = value;
    }

    public int getId() {
        return id;
    }
}
