// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.util.GregorianCalendar;
import java.util.List;
import org.apache.hivemind.ApplicationRuntimeException;

/**
 * This is the Visit class. It is used to store all information 
 * concerning a certain visit to a web application.
 */

public class Visit {

    private String username;
    private String name;
    private String password;
    private Student student;
    private Student monitor;
    private Exercise exercise;
    private Course course;
    private Round round;
    private ExerciseTemplate exerciseTemplate;

    private boolean requestDebug;
    private boolean disableInspector;
    private boolean warning;
    private String warningText;

    // fields relevant to monitors only
    private GregorianCalendar start;
    private GregorianCalendar end;
    private int activityViewMode;

    private boolean[] roundIndicator;
    //private boolean mainPageDetails; 
    private boolean mainPageGrading; 
    private boolean mainPageActivity;
    private boolean mainPageGradeDistribution;
    //private boolean mainPageStatistics; 

    private Statistics courseStatistics;
    private List studentResults;

    // view modes
    public static final int NONE = -1;
    public static final int DAY = 0;
    public static final int WEEK = 1;
    public static final int COURSE = 2;
    public static final int DAY_AVERAGE = 3;

    public Visit() {
        monitor = null;
        //throw new ApplicationRuntimeException("Creating visit");
    }

    public boolean[] getRoundIndicator() {
	if(roundIndicator == null){
	    roundIndicator = new boolean[getCourse().getRoundList().size()+1];
	}else if(roundIndicator.length != getCourse().getRoundList().size()) {
	    boolean[] oldRounds = roundIndicator;
	    roundIndicator = new boolean[getCourse().getRoundList().size()+1];
	    for(int i = 0; i < roundIndicator.length-1 && i < oldRounds.length-1; i++) {
		roundIndicator[i] = oldRounds[i];
	    }
	    roundIndicator[roundIndicator.length-1] = oldRounds[oldRounds.length-1];
	}
	return roundIndicator;
    }

    public boolean isWarning() {
	return warning;
    }

    public void setWarning(boolean value) {
	warning = value;
    }

    public String getWarningText() {
	return warningText;
    }

    public void setWarningText(String text) {
	warningText = text;
    }

    public boolean isRequestDebug() {
	return requestDebug;
    }

    public void setRequestDebug(boolean value) {
	requestDebug = value;
    }

    public boolean isDisableInspector() {
	return disableInspector;
    }

    public void setDisableInspector(boolean value) {
	disableInspector = value;
    }

    public String getUsername() {
	return username;
    }

    public void setUsername(String value) {
	username = value;
    }

    public String getName() {
	return name;
    }

    public void setName(String value) {
	name = value;
    }

    public String getPassword() {
	return password;
    }

    public void setPassword(String value) {
	password = value;
    }

    public Student getStudent() {
	if (monitor == null) {
	    if(username == null) {
		return null;
	    }
	    return DataStorage.getInstance().getStudent(username);
	} else {
	    return monitor;
	}
    }

    public Student getMonitor() {
	return monitor;
    }

    public void setMonitor(Student s) {
	if(s.isInstructor()) {
	    monitor = s;
	}
    }

    public boolean isMonitor() {
	if(monitor != null && monitor.isInstructor())
	    return true;
	return false;
    }

    public boolean isAdmin() {
	if(monitor != null && monitor.isAdmin()) 
	    return true;
	return false;
    }

    public Student getMonitoredStudent() {
	return student;
    }

    public void setMonitoredStudent(Student value) {
  	student = value;
    }

    public Exercise getExercise() {
	return exercise;
    }

    public void setExercise(Exercise ex) {
	exercise = ex;
    }

    public Course getCourse() {
    	if(course == null) {
    		Student stu = getStudent();
    		if(stu == null)
    			return null;
    		if (stu.getDefaultCourse() != null) {
    		 	setCourse(stu.getDefaultCourse());
    		} else {
    			setCourse((Course) stu.getCourseList().get(0));
    		}
    	}
    	return course;
    }
    
    public void setCourse(Course c) {
    	//System.out.println("setCourse called with " + ((c == null) ? "null" : ""+c.getId()));
    	if(course != null && c != null && course.getId() != c.getId()) {
    		//System.out.println("Setting course to " + c.getId());
    		course = c;
    		studentResults = null;
    		courseStatistics = null;
    	}
    	if (course == null && c != null) {
    		//System.out.println("fooSetting course to " + c.getId());
    		course = c;
    		studentResults = null;
    		courseStatistics = null;
    	}
    }
    
    public void clearCourse() { course = null; }

    public Round getRound() {
	return round;
    }

    public void setRound(Round r) {
	round = r;
    }

    public void setExerciseTemplate(ExerciseTemplate et) {
	exerciseTemplate = et;
    }

    public ExerciseTemplate getExerciseTemplate() {
	return exerciseTemplate;
    }

    public GregorianCalendar getStart() {
	return start;
    }

    public void setStart(GregorianCalendar d) {
	start = d;
    }

    public GregorianCalendar getEnd() {
	return end;
    }

    public void setEnd(GregorianCalendar d) {
	end = d;
    }

    public int getActivityViewMode() {
	return activityViewMode;
    }

    public void setActivityViewMode(int i) {
	activityViewMode = i;
    }

    public void clearFields() {
	username = null;
	name = null;
	password = null;
	student = null;
	monitor = null;
	exercise = null;
	course = null;
	round = null;
	exerciseTemplate = null;
	warning = false;
	warningText = null;
	start = null;
	end = null;
	activityViewMode = NONE;
	roundIndicator = null;
    }

//      public boolean isDetails() {
//  	return mainPageDetails;
//      }

    public boolean isGrading() {
	return mainPageGrading;
    }

    public boolean isGradeDistribution() {
	return mainPageGradeDistribution;
    }

    public boolean isActivity() {
	return mainPageActivity;
    }

//      public boolean isShowStatistics() {
//  	return mainPageStatistics;
//      }

    public Statistics getStatistics() {
	if(courseStatistics == null) {
	    DataStorage ds = DataStorage.getInstance();
	    courseStatistics = ds.getStatisticsForCourse(getCourse().getId());
	}
	return courseStatistics;
    }

//      public void setDetails(boolean b) {
//  	mainPageDetails = b;
//      }

    public void setGrading(boolean b) {
	mainPageGrading = b;
    }

    public void setGradeDistribution(boolean b) {
	mainPageGradeDistribution = b;
    }

    public void setActivity(boolean b) {
	mainPageActivity = b;
    }

//      public void setShowStatistics(boolean b) {
//  	mainPageStatistics = b;
//      }

    public List getStudentResults() {
	if(studentResults == null) {
	    DataStorage ds = DataStorage.getInstance();
	    studentResults = ds.getCourseResults(getCourse());
	}
	return studentResults;
    }

}
