// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.util.Locale;
import org.apache.tapestry.IRequestCycle;

import cexer.*;
import cexer.util.Checker;
import cexer.util.Logging;

/** Provides dynamic behaviour for the Login page */
public abstract class LoginPage extends SecurePage {
    private String username;

    //private final String ANONYMOUS = "00000A";

    private int courseId = -1;

    private String password;

    /**
     * Returns true, if the course id has been given as parameter, false
     * otherwise.
     */
    public boolean isParameter() {
        return checkParameter(getRequestCycle().getRequestContext().getParameter("course"));
    }

    public abstract boolean isWarning();

    public abstract void setWarning(boolean b);

    public abstract String getWarningText();

    public abstract void setWarningText(String s);

    public String getUsername() {
        return username;
    }

    public void setUsername(String value) {
        username = value;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String value) {
        password = value;
    }

    /**
     * Fetches the injected student-session object (see Login.page).
     */
    public abstract StudentSession getStudentSession();


    /**
     * @param cycle
     */
    public void formSubmit(IRequestCycle cycle) {
        if (getUsername() == null) { // added this check to assure compatibility with tapestry 4
            LoginPage next_page = (LoginPage) cycle.getPage("Login");
            next_page.setWarning(true);
            next_page.setWarningText(getMessage("error-message"));
            next_page.setPassword("");
            cycle.activate(next_page);
            return;
        }

        String studentNumber = getUsername().toUpperCase();
        DataStorage ds = DataStorage.getInstance();

        if (!ds.hasRegistrations(studentNumber)) {
            // student.getCourseList().isEmpty()) {
            // no registrations -> Register

            // If the course has been given as a parameter, set it to visit
            // before going to registration page

            if (courseId != -1) {
                Course c = DataStorage.getInstance().getCourse(courseId);
                if (c != null) {
                    StudentSession session = getStudentSession();
                    session.clearFields();
                    session.setCourse(c);
                }
            }

            cexer.util.Logging.info(this, "No registrations.");

            LoginPage next_page = (LoginPage)cycle.getPage("Login");
            next_page.setWarning(true);
            next_page.setWarningText(getMessage("error-message"));
            next_page.setPassword("");
            // cycle.activate("Register"); // uncomment this is registration
            // should be shown
            cycle.activate(next_page);
            return;
        }

        Student s = ds.getStudent(studentNumber);
        if (s == null || s.getPassword() == null
                || !s.matchPassword(getPassword())) {

            LoginPage next_page = (LoginPage) cycle.getPage("Login");
            next_page.setWarning(true);
            next_page.setWarningText(getMessage("error-message"));
            next_page.setPassword("");
            // next_page.setUsername(null);
            cexer.util.Logging.info(this, "Student " + studentNumber + " tried to log in with invalid password.");
            cycle.activate(next_page);
            return;
        }

        if(s.getAllCourses().size() == 0) {
            LoginPage next_page = (LoginPage) cycle.getPage("Login");
            next_page.setWarning(true);
            next_page.setWarningText(getMessage("no-courses-error"));
            next_page.setPassword("");            
            cexer.util.Logging.info(this, "Student " + studentNumber + " tried to log in, but is not registered to any courses (as student).");
            cycle.activate(next_page);
            return;
        }

        StudentSession session = getStudentSession();
        session.clearFields();
        //session.setWarning(false);
        session.setIsLoggedIn(true);
        session.setAuthMethod(StudentSession.PASSWORD);
        session.setUserId(studentNumber);
        cycle.getEngine().setLocale(new Locale(s.getLanguage()));
        cexer.util.Logging.info(this, "Student " + studentNumber + " logged in");
        cycle.activate("Main");
    }

    /* XXX Not up-to-date */
    /*
     * public void anonymousLoginAction(IRequestCycle cycle) { Visit visit =
     * (Visit)getVisit(); visit.setWarning(false); DataStorage ds =
     * DataStorage.getInstance(); Student student = ds.getStudent(ANONYMOUS);
     * //visit.setStudent(student); visit.setUsername(ANONYMOUS);
     * cycle.activate("Main"); }
     */

    public void registrationAction(IRequestCycle cycle) {

        // If the course has been given as a parameter, set it to visit
        // before going to registration page
        if (courseId != -1) {
            Course c = DataStorage.getInstance().getCourse(courseId);
            if (c != null) {
                StudentSession session = getStudentSession();
                session.setCourse(c);
            }

        }
        // This is why PageLink is not used!
        //session.setWarning(false);
        cycle.activate("Register");
    }

    // public void adminLoginAction(IRequestCycle cycle) {
    // cycle.activate("AdminLogin");
    // }

    public void detach() {
        super.detach();
        username = null;
        password = null;
    }

    /**
     * Returns true, if the course id has been given as parameter, false
     * otherwise and also handles the parameter.
     */
    private boolean checkParameter(String courseId) {
        if (courseId != null)
            try {
                this.courseId = Integer.parseInt(courseId);
            } catch (NumberFormatException e) {
                Logging.error(this, "Invalid course id as parameter: "
                        + courseId);
            }
        return (this.courseId != -1);
    }

    public String getMessageText() {
        String msg = DataStorage.getInstance().getMessage(
                getPage().getLocale().getLanguage());
        return msg;
    }
    
}
