// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import org.apache.tapestry.IRequestCycle;

import cexer.DataStorage;
import cexer.Student;
import cexer.StudentSession;
import cexer.util.PasswordService;

/**
 * Page used for generating a new password for a user. The user
 * enters the page by following a link from a email that he has received.
 * 
 * @author Erik Fallenius
 *
 */
public abstract class RecoverPasswordPage extends SecurePage {

    /**
     * Gets the student session state object.
     * 
     * @return student session object
     */
    public abstract StudentSession getStudentSession();
    
    /**
     * Sets the state of the page. True means that the password has been generated.
     * 
     * @param state the state
     */
    public abstract void setGenerated(boolean state);
    
    /**
     * Sets the generated password.
     * 
     * @param password the password to set
     */
    public abstract void setGeneratedPassword(String password);
    
    
    /**
     * Gets the user id entered by the user
     * 
     * @return user id
     */
    public abstract String getEnteredUserId();
    
    /**
     * Sets the state of the warning dialog
     * 
     * @param state true shows the dialog, false hides it
     */
    public abstract void setWarning(boolean state);
    
    /**
     * Sets the text of the warning dialog
     * 
     * @param text the warning text
     */
    public abstract void setWarningText(String text);
    
    /**
     * Tells if page access is allowed.
     * 
     * @return true if allowed, false otherwise
     */
    public boolean isHasAccess() {
        if(getStudentSession().getUserId() == null) {
            String pageId = getPage().getRequestCycle().getParameter("id");
            PasswordService ps = PasswordService.getInstance();
            DataStorage ds = DataStorage.getInstance();        
            String userId = "";
            if(pageId != null)
                userId = ds.getRecoveryPageData(ps.encrypt(pageId));
            Student s = ds.getStudent(userId);
            if(s != null)
                getStudentSession().setUserId(userId);            
        }

        if(getStudentSession().getUserId() != null)
            return true;
        else return false;
    }
    
    /**
     * Gets the user id.
     * 
     * @return user id
     */
    public String getUserId() {
        return getStudentSession().getUserId();
    }
    
    /**
     * Generates a new password and stores it in the database.
     * 
     * @param cycle the <code>IRequestCycle</code>
     */
    public void generatePassword(IRequestCycle cycle) {
        if(getEnteredUserId() == null || getEnteredUserId() == "" || 
                (!getUserId().equals(getEnteredUserId()) && !getUserId().equals(getEnteredUserId().toUpperCase()))) {
            setWarning(true);
            setGenerated(false);
            setWarningText("The entered user name doesn't match.");
            cycle.activate(this);            
        } else {
            setWarning(false);
            PasswordService ps = PasswordService.getInstance();
            String generatedPassword = ps.generatePassword(8);
            DataStorage ds = DataStorage.getInstance();        
            ds.setStudentField(getStudentSession().getUserId(), "password", ps.encrypt(generatedPassword));
            ds.removeRecoveryPageData(getStudentSession().getUserId());
            setGenerated(true);
            setGeneratedPassword(generatedPassword);
            cycle.activate(this);
        }
    }
    
}
