// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.valid.ValidationDelegate;
import org.apache.tapestry.event.PageAttachListener;
import org.apache.tapestry.event.PageValidateListener;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.services.RequestGlobals;
import org.apache.tapestry.annotations.InjectObject;
import org.apache.tapestry.PageRedirectException;
import cexer.*;
import cexer.util.Checker;
import cexer.util.CourseSelectionModel;
import cexer.util.Logging;
import cexer.util.LocaleSelectionModel;
import cexer.util.PasswordService;

public abstract class RegisterPage extends SecurePage implements PageAttachListener, PageValidateListener {

    /*
     * Needed to get the GET params
     */
    public abstract RequestGlobals getRequestGlobals();


    public void pageAttached(PageEvent event) {
        setLanguage(new Locale(Student.DEFAULT_LANGUAGE));

        // Handle the GET parameter
        String cId = getRequestGlobals().getRequest().getParameter("cid");

        if (cId != null && cId.length() > 0) {
            try {
                Course paramCourse = DataStorage.getInstance().getCourse(Integer.parseInt(cId));
                if (paramCourse != null) {
                    setCourse(paramCourse);
                }
            } catch (NumberFormatException e) {
                Logging.error(this, "Invalid course id as a parameter: " + cId);
            }
        }
    }

    /**
     * Checks that the course is open. User is redirected to an error page if this is not the case.
     */
    public void pageValidate(PageEvent event) {
        // NOTE: setters are called after this, so we must have another check in formSubmit

        Course course = getCourse();

        if (course != null && !course.isOpen()) {
            ErrorPage errorPage = getErrorPage();
            errorPage.setHeading("Registration for " + course + " is not open");
            Logging.info(this, "Registration closed: " + course);
            //errorPage.setText("Contact course staff.");

            throw new PageRedirectException(errorPage);
        }
    }


    public void formSubmit(IRequestCycle cycle) {
        //handleParam();
        //System.out.println("formsubmit: course: " + getCourse());
        setWarning(true);

        // Don't proceed if we have errors
        ValidationDelegate delegate = (ValidationDelegate)getBeans().getBean("delegate");
        if (delegate.getHasErrors()) {
            //setCourse(null);
            setPassword("");
            setPassword2("");
            cycle.activate(this);

            return;
        }

        DataStorage ds = DataStorage.getInstance();
        String studentNumber = getUserId().toUpperCase();
        Student student = ds.getStudent(studentNumber);
        String checker = getCourse().getCheckerClass();

        if(!Checker.check(studentNumber, checker)) {
            setWarningText("Invalid student number for the selected course.");
            return;
        }

        if (!getPassword().equals(getPassword2())) {
            setWarningText("Passwords do not match");
            return;
        }

        if (getPassword().length() > 12) {
            setWarningText("Password too long (max 12 characters).");
            return;
        }

        if (student != null) {
            // Student exists in db

            if (!student.matchPassword(getPassword())) {
                setWarningText("Student already registered, password incorrect.");
                Logging.info(this, "Student " + student.getStudentNumber() +
                            " tried to register to a new course with incorrect password.");
                return;
            } else if (student.isOnCourse(getCourse())) {
                setWarningText("Student already registered to selected course.");
                return;
            }

            if (!student.getName().equals(getName())) {
                student.setName(getName());
                ds.setStudentField(studentNumber, "name", getName());
            }

            if (student.getEmail() == null || !student.getEmail().equals(getEmail())) {
                student.setEmail(getEmail());
                ds.setStudentField(studentNumber, "email", getEmail());
            }

            String lang = getLanguage().getLanguage();
            if (!student.getLanguage().equals(lang)) {
                student.setLanguage(lang);
                ds.setStudentField(studentNumber, "lang", lang);
            }
        } else {
            // New student
            PasswordService passwordService = PasswordService.getInstance();
            student = new Student(studentNumber, getName());
            student.setName(getName());
            student.setPassword(passwordService.encrypt(getPassword()));
            student.setEmail(getEmail());
            student.setStudentNumber(studentNumber);
            String lang = getLanguage().getLanguage();
            student.setLanguage(lang);

            boolean ok = ds.addStudent(student);
            if (!ok) {
                setWarningText("Error when adding student!");
                Logging.error(this, "Error when adding student " + studentNumber);
                return;
            }
        }

        student.addCourse(getCourse());
        student.setDefaultCourse(getCourse());
        ds.setStudentDefaultCourse(studentNumber, getCourse().getId());
        ds.registerStudentToCourse(studentNumber, getCourse().getId());
        getEngine().setLocale(getLanguage());
        StudentSession session = getStudentSession();
        session.clearFields();
        session.setIsLoggedIn(true);
        session.setAuthMethod(StudentSession.PASSWORD);
        session.setUserId(studentNumber);
        cexer.util.Logging.info("Student " + studentNumber + " registered to course "
                                + getCourse().getId());

        cycle.activate("Main");
    }

    public boolean isCourseParam() {
        return (getCourse() != null);
    }


    public void detach() {
        super.detach();
        setLanguage(new Locale(Student.DEFAULT_LANGUAGE));
    }

    /**
     * Returns a list of available courses.
     */
    public CourseSelectionModel getCourses() {
        return DataStorage.getInstance().getLocalizedCourseSelectionModel(getPage().getLocale().getLanguage());
    }

    /**
     * Returns a list of available languages.
     */
    public LocaleSelectionModel getLocaleModel() {
        LocaleSelectionModel lsm = DataStorage.getInstance().getLocaleSelectionModel();
        return lsm;
    }

    public String getLocalizedCourseName() {
        Course c = getCourse();

        if(c != null) {
            return c.getCourseCode() + " " + c.getLocalizedName(getPage().getLocale().getLanguage());
        } else {
            return "";
        }
    }

    public int getCourseId() {
        /* CourseID is stored into a hidden field. */

        Course course = getCourse();  // Course has been loaded in pageAttached()

        if (course != null) {
            return course.getId();
        } else {
            return 0;
        }
    }

    public void setCourseId(int courseId) {
        /* The hidden field uses setCourseId() to report the value back to us. */
        Logging.info(this, "** TA ** setCourseId: " + courseId);

        if (courseId < 1) {
            return;
        }

        Course paramCourse = DataStorage.getInstance().getCourse(courseId);
        if (paramCourse != null) {
            setCourse(paramCourse);
        }
    }

    public abstract ErrorPage getErrorPage();

    public abstract void setCourse(Course c);

    public abstract Course getCourse();

    public abstract StudentSession getStudentSession();

    public abstract void setPassword(String value);

    public abstract String getPassword();

    public abstract void setPassword2(String value);

    public abstract String getPassword2();

    public abstract void setEmail(String value);

    public abstract String getEmail();

    public abstract void setLanguage(Locale value);

    public abstract Locale getLanguage();

    public abstract String getName();

    public abstract void setName(String name);

    public abstract String getUserId();

    public abstract void setUserId(String userId);

    public abstract void setWarning(boolean state);

    public abstract boolean getWarning();

    public abstract void setWarningText(String warningText);

    public abstract String getWarningText();

}
