// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.util;

import java.util.Calendar;

/**
 * This is an array implementation of a small cache, that uses
 * interger keys for storage and retrieval of data. Old data
 * is removed when new with same key is inserted into cache.
 * Default cache size is 61. For good performance cache size should
 * be a prime number, since the hashing function is extremely
 * naive. (key mod cachesize)
 *
 * This is basically a hack.
 */

public class ArrayCache  {

    protected Object[] cache;
    protected int size;
    private int pos = 0;
    private Calendar expires = null;
    public static final int expirationAmount = 1;
    public static final int expirationField = Calendar.DATE;

    public ArrayCache() {
        /*cache = new Object[61];
        size = 61;*/
        this(61);
    }

    public ArrayCache(int size) {
        cache = new Object[size];
        this.size = size;
        expires = Calendar.getInstance();
        expires.add(expirationField, expirationAmount);
        expires.set(Calendar.HOUR_OF_DAY, 5);
        expires.set(Calendar.MINUTE, 0);
    }

    /**
     * Inserts a new value into the cache. Possible old value with same
     * key is removed
     */
    public synchronized void insert(Object o, int key) {
        key = Math.abs(key % size);
        cache[key] = o;
    }

    /**
     * Insert new value into the cache, where the key can be an Object 
     */
    public synchronized void insert(Object o, Object key) {
        int hv = key.hashCode();
        hv = Math.abs(hv % size);
        cache[hv] = o;
    }

    public synchronized Object get(Object key) {
        int hv = key.hashCode();
        return cache[Math.abs(hv % size)];
    }

    /**
     * Retrieves value from cache by key. 
     */
    public synchronized Object get(int key) {
        key = Math.abs(key % size);
        return cache[key];
    }

    public void remove(int key) {
        insert(null, key);
    }

    public synchronized void removeAll() {
        for(int i = 0; i < size; i++) {
            cache[i] = null;
        }

        return;
    }

    public synchronized void remove(Object o) {
        if(o == null) {
            return;
        }

        for(int i = 0; i < size; i++) {
            if(cache[i] != null && cache[i].equals(o)) {
                cache[i] = null;
                return;
            }
        }
    }

    public synchronized boolean contains(Object o) {
        if(o == null) {
            return false;
        }

        for(int i = 0; i < size; i++) {
            if(cache[i] != null && cache[i].equals(o)) {
                return true;
            }
        }

        return false;
    }

    public boolean isExpired() {
        Calendar now = Calendar.getInstance();
        return now.after(expires);
    }
}
