// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.xmlrpc;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Calendar;
import java.util.Hashtable;
import java.util.List;

import cexer.*;
import cexer.util.Logging;
import cexer.util.FingerprintHelper;

public class ResultHandler {

    /** Method that actually adds the result to the DataStorage. */
    private Hashtable addResult(String studentId, String courseId,
        int roundNumber, int exerciseNumber,
        int points, /* start experimental code */ String fileName, /* end experimental code */ int status)
    {

        Hashtable ret = new Hashtable();
        try {
            DataStorage ds = DataStorage.getInstance();
            int success = ds.setResult(studentId, courseId,
                        roundNumber, exerciseNumber,
                        points, fileName, status);

            if (success == DataStorage.OK) {
                ret.put("operationOk", new Boolean(true));
            } else {
                ret.put("operationOk", new Boolean(false));
            }

            if (success == DataStorage.STUDENT_NOT_FOUND) {
                ret.put("message", "ERROR: Student " + studentId +
                    " not found from database");
                Logging.error(this, "ERROR: Student " + studentId + " not found from database");
            } else if (success == DataStorage.COURSE_NOT_FOUND) {
                ret.put("message", "ERROR: Course " + courseId +
                    " not found from database");
            } else if (success == DataStorage.STUDENT_NOT_ON_COURSE) {
                ret.put("message", "ERROR: Student " + studentId +
                    " not registered on course " + courseId);
            } else if (success == DataStorage.DATABASE_ERROR) {
                ret.put("message", "ERROR: Error connecting to database");
            }

            return ret;
        } catch (Exception e) {
            e.printStackTrace();
            Logging.error(this, "Exception while adding result. Submission details:" +
                getDescr(studentId, courseId, roundNumber, exerciseNumber,
                    points), e);
            ret.put("operationOk", new Boolean(false));
            ret.put("message", "ERROR: " + e.getMessage());
            return ret;
        }
    }

    /** Method called through xmlrpc to add a result. The fingerprint is
     * checked and no result is added to DataStorage if it doesn't match. */
    public Hashtable addResult(String studentId, String courseId,
                int roundNumber, int exerciseNumber,
                int points, String exerciseClass,
                int maxPoints, /* start experimental code */ String fileName, /*end experimental code */
                String fingerprint)
    {
        //System.out.println("filename: " + fileName + " fingerprint: " + fingerprint);
        //System.out.println("" + courseId + " " + roundNumber + " " + exerciseNumber + " " + studentId + " " + maxPoints + " " + exerciseClass);
        String md5 = "";
        DataStorage ds = DataStorage.getInstance();

        try {
            md5 = FingerprintHelper.calculateMD5(FingerprintHelper.getFingerprint(courseId, roundNumber,
                                exerciseNumber, studentId,
                                maxPoints, exerciseClass));
        } catch (NoSuchAlgorithmException e) {
            Logging.error(this, "Exception while adding result. Submission details: " +
                getDescr(studentId, courseId, roundNumber, exerciseNumber, points), e);

            Hashtable ret = new Hashtable();
            ret.put("operationOk", new Boolean(false));
            ret.put("message", "ERROR: " + e.getMessage());

            return ret;
        }

        try {
            Student st = DataStorage.getInstance().getStudent(studentId);
            if(st == null) {
                Hashtable ret = new Hashtable();
                ret.put("operationOk", new Boolean(false));
                ret.put("message", "Error: student not found");
                return ret;
            }

            Course c = DataStorage.getInstance().getCourse(Integer.parseInt(courseId));
            if(String.valueOf(DataStorage.DUMMY_COURSE).equals(courseId)) {
                Hashtable ret = new Hashtable();
                ret.put("operationOk", new Boolean(true));
                ret.put("message", "Dummy submission");
                return ret;
            }

            if(c == null) {
                Hashtable ret = new Hashtable();
                ret.put("operationOk", new Boolean(false));
                ret.put("message", "Error: no course found");
                return ret;
            }

            int dlStatus = isBeforeDeadLine(courseId, roundNumber, st);
            Round r = getRound(c, roundNumber);

            if(r == null) {
                Hashtable ret = new Hashtable();
                ret.put("operationOk", new Boolean(false));
                ret.put("message", "Error: Round not found");
                return ret;
            }

            List l = r.getExerciseList();
            Exercise e;

            if(l != null) {
                e = (Exercise)r.getExerciseList().get(exerciseNumber - 1);
            } else {
                Hashtable ret = new Hashtable();
                ret.put("operationOk", new Boolean(false));
                ret.put("message", "Error: Exercise not found");
                return ret;
            }

            if (!fingerprint.equals(md5)) {
                Logging.warn(this, "Submission with invalid fingerprint: " +
                        studentId + " - " + courseId + "-" + roundNumber +
                        "-" + exerciseNumber + " points: " + points);
                int i = DataStorage.getInstance().setResult(studentId, courseId,
                                        roundNumber, exerciseNumber,
                                        points, fileName, DataStorage.INVALID_FINGERPRINT);

                if (i != DataStorage.OK) {
                    Logging.error(this, "Coudn't add the invalid fingerprint " +
                        "result to database. Submission details: " +
                        getDescr(studentId, courseId, roundNumber, exerciseNumber,
                            points) + "Error code was " + i);
                }

                Hashtable ret = new Hashtable();
                ret.put("operationOk", new Boolean(false));
                ret.put("message", "ERROR: Invalid fingerprint!");

                return ret;
            } else if (dlStatus == DataStorage.AFTER_CLOSING) {
                int i = DataStorage.getInstance().setResult(studentId, courseId,
                                        roundNumber, exerciseNumber,
                                        points, fileName, DataStorage.AFTER_CLOSING);
                if (i != DataStorage.OK) {
                    Logging.error(this, "Coudn't add the submission after closing " +
                        "result to database. Submission details: " +
                        getDescr(studentId, courseId, roundNumber, exerciseNumber,
                            points) + "Error code was " + i);
                }

                Hashtable ret = new Hashtable();
                ret.put("operationOk", new Boolean(false));
                ret.put("message", "ERROR: Course closed");

                return ret;
            } else if (st.getResults(c).getSubmissions(roundNumber, exerciseNumber) >= e.getSubs()) {
                int i = DataStorage.getInstance().setResult(studentId, courseId,
                                        roundNumber, exerciseNumber,
                                        points, fileName, DataStorage.ALL_SUBMISSIONS_USED);

                if (i != DataStorage.OK) {
                    Logging.error(this, "Coudn't add the submission after closing " +
                        "result to database. Submission details: " +
                        getDescr(studentId, courseId, roundNumber, exerciseNumber,
                            points) + "Error code was " + i);
                }

                Hashtable ret = new Hashtable();
                ret.put("operationOk", new Boolean(i == DataStorage.OK));
                ret.put("message", "WARNING: You have used your allowed submissions for this exercise.\nThis submission will not be used in grading.\n");

                return ret;
            } else if (dlStatus == DataStorage.AFTER_DEADLINE) {
                int makeupPoints = (int)Math.ceil(points*r.getMakeupFactor());
                Hashtable ret = addResult(studentId, courseId, roundNumber, exerciseNumber, points, fileName, DataStorage.AFTER_DEADLINE);

                if (((Boolean)ret.get("operationOk")).booleanValue()) {
                    ret.put("message", "Deadline for the exercise has passed, points were reduced.\nStudent " + studentId + " on course " + courseId + " submitted exercise " + roundNumber + "." + exerciseNumber +". Grade=" + makeupPoints);
                }

                return ret;
            } else {
                return addResult(studentId, courseId, roundNumber, exerciseNumber, points, fileName, DataStorage.OK);
            }

        } catch (Exception e) {
            e.printStackTrace();
            Logging.error(this, "Exception while adding result. Submission details:" +
                getDescr(studentId, courseId, roundNumber, exerciseNumber, points), e);

            Hashtable ret = new Hashtable();
            ret.put("operationOk", new Boolean(false));
            ret.put("message", "ERROR: " + e.getMessage());

            return ret;
        }

    }

    private int isBeforeDeadLine(String courseId, int round, Student s) {
        Course c = DataStorage.getInstance().getCourse(Integer.parseInt(courseId));
        Round r = getRound(c, round);

        if (r != null) {
            Calendar deadLine = Calendar.getInstance();
            if (s.hasPersonalDeadLine(r.getId()))
            deadLine.setTime(s.getDeadLine(r.getId()));
            else
            deadLine.setTime(r.getDeadLine());
            if (deadLine.getTime().after(Calendar.getInstance().getTime())) {
                return DataStorage.OK;
            } else { //if (c.getClose().after(Calendar.getInstance().getTime()))
                return DataStorage.AFTER_DEADLINE;
            }
            //else {
            //    return DataStorage.AFTER_CLOSING;
            //}
        } else {
            return DataStorage.ROUND_NOT_FOUND; //should never happen
        }
    }

    private Round getRound(Course c, int round) {
        List l = c.getRoundList();
        Round r = null;
        int i = 0;
        while (i < l.size()) {
            if (((Round)l.get(i)).getRoundNumber() == round) {
                r = (Round)l.get(i);
                break;
            }
            i++;
        }
        return r;
    }

    private String getDescr(String studentId, String courseId, int roundNumber,
                            int exerciseNumber, int points)
    {
        return "Submission of " + studentId + " for course " + courseId + " exercise "
                + roundNumber + "." + exerciseNumber + " with points " + points;
    }
}
