#!/bin/bash

JAVALIBS_OK='true'
TRAKLA_DIR=`pwd`
LIB_DIR=$TRAKLA_DIR/code/lib
DOWNLOAD_DIR=$TRAKLA_DIR/libs
USE_COLORS='true'

# Set editor
if [ -z $EDITOR ] ; then
    EDITOR=`which nano`
fi
#echo "Using $EDITOR as editor"


function check_software() {
    retval=0

    # Check wget
    if [ -z `which wget` ] ; then
        echo -e $red'ERROR:'$normal 'wget is missing. It is needed for automatic downloading of libraries.'
        echo 'Please install wget or follow the instructions in the installation guide to download software manually.'
        retval=1
    fi

    # Check ant
    if [ -z `which ant` ] ; then
        echo -e $red'ERROR:'$normal "ant is missing. It is needed for building Trakla2 and it's dependencies."
        retval=1
    fi

    return $retval
}


##
# Copies jarFilename from $DOWNLOAD_DIR to $LIB_DIR if it isn't already there. If the
# jar is not found from libs either, JAVALIBS_OK is set to false and message is
# shown to user.
# install_jar  jarFilename displayName message
#
function install_jar_manual() {
    # Is the file already installed?
    if [ ! -f $LIB_DIR/$1 ]; then
        # Do we have the file in libs?
        if [ -f $DOWNLOAD_DIR/$1 ]; then
            echo "Copying $2"
            cp $DOWNLOAD_DIR/$1 $LIB_DIR/
        else
            echo ""
            echo -e $red'***'$normal "$2 is missing." $red'***'$normal
            echo $3
            echo ""
            JAVALIBS_OK='false'
        fi
    else
        echo -e "$2 already installed." $green'Skipping.'$normal
    fi
}


##
# install_jar  targetJar url
# If 'targetJar' is missing, 'url' is downloaded and extracted, and all jar files in it are copied to $LIB_DIR.
# This function can handle tar.gz, tgz and .zip packages.
#
function install_jar() {
    # If 'targetJar' does not exist, download the package
    if [ ! -f $LIB_DIR/$1 ] ; then
        download_tar $2

        cp -v $DIRNAME/*.jar $LIB_DIR
    else
        echo -e "$1 already installed." $green'Skipping.'$normal
    fi
}


##
# install_tar  url [directoryName]
# 'url' is downloaded and unzipped unless a matching package or directory already exists.
# Name of the exraxted directory is guessed from the name of the zip unless it is given as a parameter.
#
function download_tar() {
    # Get filename from url
    TARNAME=`basename $1`

    # Detect file name exetnsion. Get directory name by removing the extension.
    if [[ ${TARNAME: -7} = ".tar.gz" ]] ; then
        DIRNAME=${TARNAME%\.tar\.gz}
    fi

    if [[ ${TARNAME: -4} = ".tgz" ]] ; then
        DIRNAME=${TARNAME%\.tgz}
    fi

    if [[ ${TARNAME: -4} = ".zip" ]] ; then
        DIRNAME=${TARNAME%\.zip}
    fi

    # Override DIRNAME if requested
    if [[ -n $2 ]] ; then
        DIRNAME=$2
    fi

    # If the source directory does not exist, extract tar
    if [ ! -d $DIRNAME ] ; then
        # If the tar does not exist, download it
        if [ ! -f $TARNAME ] ; then
            wget $1
            # TODO: show an error message if this fails
        else
            echo -e $TARNAME 'already downloaded.' $green'Skipping.'$normal
        fi

        # Extract
        if [[ ${TARNAME: -4} = ".zip" ]] ; then
            unzip -q $TARNAME
        else
            tar -xzf $TARNAME
        fi
    else
        echo -e "$DIRNAME already extracted." $green'Skipping.'$normal
    fi
}


#
# Downloads and builds Tapestry and it's dependencies
# Note: Working directory is changed.
#
function install_tapestry() {
    cd $DOWNLOAD_DIR

    # If tapestry has not been built, build it
    if [ ! -f tapestry-4.0.2/ext-package/lib/tapestry-4.0.2.jar ] ; then
        # Download tapestry
        download_tar http://mirror.eunet.fi/apache/tapestry/tapestry-4.0.2.tar.gz

        # Download hivemind
        download_tar http://mirror.eunet.fi/apache/hivemind/hivemind-1.1.1.tar.gz

        # Adjust build.properties of Tapestry
        echo "hivebuild.dir=`pwd`/hivemind-1.1.1/hivebuild" > tapestry-4.0.2/config/build.properties

        cd tapestry-4.0.2

        # FIXME: ant tapestry tries to install JUnit, which will fail if this script is not being run as root.
        # Perhaps junit should be installed manually before running this script.
        ant install
        if [ $? -ne 0 ]; then
            echo ""
            echo -e $red'ERROR'$normal ' Failed to build tapestry'
            echo "Most likely tapestry was trying to install JUnit, which cannot be done without being root. Install JUnit manually and rerun this installation script."
            echo ""
            return 1
        fi

        ant install
        if [ $? -ne 0 ]; then
            echo -e $red'ERROR'$normal ' Failed to build tapestry'
            return 1
        fi

        # Remove jcharts-0.6.0.jar because we are using a newer version
        rm tapestry-4.0.2/ext-package/lib/jcharts-0.6.0.jar

        # Patch javassist
        patch_javassist
        if [ $? -ne 0 ]; then
            return 1
        fi
    else
        echo -e 'Tapestry has already been built.' $green'Skipping.'$normal
    fi

    return 0
}

# Downloads and installs Matrix. NOTE: Working directory is changed.
function install_matrix() {
    matrix_dir=$TRAKLA_DIR/Matrix-1.3

    # Download matrix
    cd $TRAKLA_DIR
    download_tar http://www.cs.hut.fi/Research/MatrixPro/release/v1_3/matrixpro-1.3-src.zip MatrixPro-1.3
    if [ -f MatrixPro-src-1.3.zip ] ; then
        rm MatrixPro-src-1.3.zip
    fi

    # Get XML-RPC
    if [ ! -f $matrix_dir/not-in-release/ext-libs/xmlrpc-1.2-b1.jar ] ; then
        cd $DOWNLOAD_DIR
        download_tar http://mirror.eunet.fi/apache/ws/xmlrpc/binaries/xmlrpc-1.2-b1.tar.gz

        cp -v $DOWNLOAD_DIR/xmlrpc-1.2-b1/*.jar $matrix_dir/not-in-release/ext-libs/
    fi

    # Copy specific.properties.base to specific.properties
    cd $matrix_dir
    if [ ! -f code/applications/trakla2/server/server.properties ] ; then
        cp code/applications/trakla2/server/server.properties.base code/applications/trakla2/server/server.properties

        # Configure Matrix
        #$EDITOR code/applications/trakla2/server/server.properties
        echo 'trakla2.server.DATADIR='$TRAKLA_DIR/submissions >> code/applications/trakla2/server/server.properties
    fi

    # Compile
    if [ ! -f $matrix_dir/code/matrix.jar ] ; then
        cd $matrix_dir/code
        echo -e 'Building Matrix'
        ant trakla2-jar
    else
        echo -e 'Matrix has already been built.' $green'Skipping.'$normal
    fi

    # Make a symbolic link
    if [ ! -f $TRAKLA_DIR/code/context/matrix.jar ] ; then
        ln -s $matrix_dir/code/matrix.jar $TRAKLA_DIR/code/context/matrix.jar
    fi
}

#
# Note: Working directory is changed.
function patch_javassist() {
    cd $DOWNLOAD_DIR

    if [ -f tapestry-4.0.2/ext-package/lib/javassist-3.4.jar ] ; then
        echo 'Javassist already patched.' $green'Skipping.'$normal
        return
    fi

    echo 'Patching javassist. See http://issues.apache.org/jira/browse/TAPESTRY-964?page=com.atlassian.jira.plugin.system.issuetabpanels:all-tabpanel'
    echo 'for further information.'

    # Patch javassist
    download_tar http://surfnet.dl.sourceforge.net/sourceforge/jboss/javassist-3.4.zip

    cd javassist-3.4/src/main/javassist
    patch < $TRAKLA_DIR/patch.javassist.ClassPool.java
    cd $DOWNLOAD_DIR/javassist-3.4

    # Build
    ant
    if [ $? -ne 0 ]; then
        echo -e $red'ERROR'$normal ' Failed to build javassist. The patching probably went wront. Perform the patch manually and rerun this installation script.'
        echo 'Make sure to name the jar javassist-3.4.jar and place it in tapestry-4.0.2/ext-package/lib/'
        return
    fi

    cd $DOWNLOAD_DIR

    rm tapestry-4.0.2/ext-package/lib/javassist-3.0.jar
    cp -v javassist-3.4/javassist.jar tapestry-4.0.2/ext-package/lib/javassist-3.4.jar
}

function cleanup() {
    # TODO
    cd $DOWNLOAD_DIR

    #rm hivemind-1.1.1.tar.gz
    ##rm -r hivemind-1.1.1
}


#
# Sets text color
# set_color color
# possible colors are: default, red, green, yellow
function set_colors() {
    if [ $USE_COLORS == 'true' ] ; then
        black='\E[01;30m'
        red='\E[01;31m'
        green='\E[01;32m'
        yellow='\E[01;33m'
        blue='\E[01;34m'
        magenta='\E[01;35m'
        cyan='\E[01;36m'
        white='\E[01;37m'
        normal='\E[0m'
    fi
}

set_colors

# Check that all the necessary software needed by this script is installed
check_software
if [ $? -eq 1 ] ; then
    exit
fi


# Create directories
cd $TRAKLA_DIR

if [ ! -d $LIB_DIR ]; then
    mkdir $LIB_DIR
fi

if [ ! -d $DOWNLOAD_DIR ]; then
    mkdir $DOWNLOAD_DIR
fi

if [ ! -d $TRAKLA_DIR/submissions ] ; then
    mkdir $TRAKLA_DIR/submissions
fi


# Check libraries that must be installed manually
install_jar_manual "mail.jar" "Java mail" "Copy mail.jar from j2ee libs to $LIB_DIR"
install_jar_manual "activation.jar" "Java Beans Activation Framework" "Copy activation.jar from j2ee libs to $LIB_DIR"
#install_jar_manual "servlet-api.jar" "servlet-api.jar" "Copy servlet-api.jar from tomcat libs to $LIB_DIR"
install_jar_manual "j2ee.jar" "j2ee.jar" "Copy j2ee.jar from j2ee libs to $LIB_DIR"
install_jar_manual "*jdbc*jar" "JDBC driver" "Download the appropriate driver from http://jdbc.postgresql.org/download.html and put it to $LIB_DIR"

if [ "$JAVALIBS_OK" == 'false' ]; then
    exit
fi


# Install tapestry
install_tapestry
if [ $? -eq 1 ]; then
    exit
fi


# Download libraries needed by Trakla
# TODO: Check those sourceforge mirrors. Some are really slow.
cd $DOWNLOAD_DIR
install_jar ehcache-1.3.0.jar http://kent.dl.sourceforge.net/sourceforge/ehcache/ehcache-1.3.0.tgz
install_jar jCharts-0.7.5.jar http://dfn.dl.sourceforge.net/sourceforge/jcharts/jCharts-0.7.5.zip
install_jar xmlrpc-2.0.jar http://mirror.eunet.fi/apache/ws/xmlrpc/binaries/xmlrpc-2.0.tar.gz


# Adjust trakla properties
cd $TRAKLA_DIR

if [ ! -f code/config/specific.properties ]; then
    cp code/config/specific.properties.base code/config/specific.properties
    echo "root.dir=$DOWNLOAD_DIR/tapestry-4.0.2" >> code/config/specific.properties
fi

if [ ! -f code/config/log4j.properties ]; then
    cp code/config/log4j.properties.base code/config/log4j.properties

    echo
    echo 'Now we must configure properties of log4j in code/config/log4j.properties'
    echo $EDITOR 'is used for editing (this can be changed by setting the EDITOR environment variable)'
    echo 'press enter to continue'
    read

    $EDITOR code/config/log4j.properties
fi

if [ ! -f code/src/cexer/DataStorage.properties ]; then
    cp code/src/cexer/DataStorage.properties.base code/src/cexer/DataStorage.properties

    echo
    echo 'Next, we are going to configure properties in trakla2/code/src/cexer/DataStorage.properties'
    echo $EDITOR 'is used for editing (this can be changed by setting the EDITOR environment variable)'
    echo 'press enter to continue'
    read
    $EDITOR code/src/cexer/DataStorage.properties
fi


# Install Matrix
install_matrix


# Build Trakla
cd $TRAKLA_DIR/code/
ant install

echo ""
if [ $? ]; then
    echo "Trakla2 was built successfully"
    echo "copy build/cexer.var to Tomcat's webapps directory"
else
    echo $red'***'$normal 'Failed to build Trakla2' $red'***'$normal
fi
